from typing import List, Tuple

import mrcfile
import numpy as np
import pooch
from napari.types import LayerDataTuple

from surforama.io.mesh import read_obj_file

_thylakoid_registry = pooch.create(
    path=pooch.os_cache("cellcanvas"),
    base_url="doi:10.5281/zenodo.10814409",
    registry={
        "S1_M3b_StII_grow2_1_mesh_data.mrc": "md5:a6e34bbf4edc767aa6c2c854c81c9c97",
        "S1_M3b_StII_grow2_1_mesh_data.obj": "md5:63b7d681204d7d3a3937154a0f4d7fc1",
        "S1_M3b_StII_grow2_1_mesh_data_seg.mrc": "md5:d88460eb3bdf3164be6053d281fc45be",
        "S1_M3c_StOI_grow2_1_mesh_data.mrc": "md5:296fbc48917c2baab7784a5ede6aae70",
        "S1_M3c_StOI_grow2_1_mesh_data.obj": "md5:076e6e8a825f67a24e28beba09edcf70",
        "S1_M3c_StOI_grow2_1_mesh_data_seg.mrc": "md5:878d4b3fc076dfc01e788cc08f9c9201",
    },
)

_covid_registry = pooch.create(
    path=pooch.os_cache("cellcanvas"),
    base_url="doi:10.5281/zenodo.10837518",
    registry={
        "TS_004_dose-filt_lp50_bin8.rec": "md5:31914b3aa32c5656acf583f08a581f64",
        "TS_004_dose-filt_lp50_bin8_membrain_model.obj": "md5:9e67ab9493096ce6455e244ca6b20220",
    },
)


def thylakoid_membrane() -> (
    Tuple[np.ndarray, Tuple[np.ndarray, np.ndarray, np.ndarray]]
):
    """Fetch the thylakoid membrane sample data.

    Data originally from Wietrzynski and Schaffer et al., eLife, 2020.
    https://doi.org/10.7554/eLife.53740
    """
    # get the tomogram
    tomogram_path = _thylakoid_registry.fetch(
        "S1_M3b_StII_grow2_1_mesh_data.mrc", progressbar=True
    )
    tomogram = mrcfile.read(tomogram_path).astype(float)

    # get the mesh
    mesh_path = _thylakoid_registry.fetch(
        "S1_M3b_StII_grow2_1_mesh_data.obj", progressbar=True
    )
    mesh_data = read_obj_file(mesh_path)

    return tomogram, mesh_data


def _thylakoid_membrane_sample_data_plugin() -> List[LayerDataTuple]:
    """napari sample data plugin for thylakoid membrane dataset."""

    # get the data
    tomogram, mesh_data = thylakoid_membrane()

    return [
        (tomogram, {"name": "tomogram"}, "image"),
        (mesh_data, {"name": "mesh"}, "surface"),
    ]


def covid_membrane() -> (
    Tuple[np.ndarray, Tuple[np.ndarray, np.ndarray, np.ndarray]]
):
    """Fetch the covid membrane sample data.

    Data originally from Ke et al., Nature, 2020.
    https://doi.org/10.5281/zenodo.10837519
    """
    # get the tomogram
    tomogram_path = _covid_registry.fetch(
        "TS_004_dose-filt_lp50_bin8.rec", progressbar=True
    )
    tomogram = mrcfile.read(tomogram_path).astype(float)

    # get the mesh
    mesh_path = _covid_registry.fetch(
        "TS_004_dose-filt_lp50_bin8_membrain_model.obj", progressbar=True
    )
    mesh_data = read_obj_file(mesh_path)

    return tomogram, mesh_data


def _covid_membrane_sample_data_plugin() -> List[LayerDataTuple]:
    """napari sample data plugin for thylakoid membrane dataset."""

    # get the data
    tomogram, mesh_data = covid_membrane()

    return [
        (tomogram, {"name": "tomogram"}, "image"),
        (mesh_data, {"name": "mesh"}, "surface"),
    ]



from typing import Callable, List

from napari.types import LayerDataTuple

from surforama.io import read_obj_file


def mesh_reader_plugin(path: str) -> Callable[[str], LayerDataTuple]:
    if isinstance(path, str) and path.endswith(".obj"):
        return obj_reader

    # format not recognized
    return None


def obj_reader(path: str) -> List[LayerDataTuple]:
    mesh_data = read_obj_file(path)
    return [(mesh_data, {}, "surface")]



from typing import Optional

import typer
from typing_extensions import Annotated

app = typer.Typer(
    help="Surforama: wiew tomogram densities on a surface.",
    no_args_is_help=True,
)


@app.command()
def launch_surforama(
    image_path: Annotated[
        Optional[str], typer.Option(help="Path to the image to load.")
    ] = None,
    mesh_path: Annotated[
        Optional[str], typer.Option(help="Path to the mesh to load.")
    ] = None,
    demo: Annotated[
        bool, typer.Option("--demo", help="launch surforama with sample data")
    ] = False,
):
    if demo and (image_path is not None or mesh_path is not None):
        raise ValueError(
            "Please do not specify an image/mesh path when launching in demo mode."
        )

    # delay imports
    import mrcfile
    import napari

    from surforama.app import QtSurforama
    from surforama.io.mesh import read_obj_file

    viewer = napari.Viewer(ndisplay=3)

    if demo:
        # set up the viewer in demo mode
        from surforama.data._datasets import thylakoid_membrane

        # fetch the data
        tomogram, mesh_data = thylakoid_membrane()

        # Add the data to the viewer
        volume_layer = viewer.add_image(
            tomogram, blending="translucent", depiction="plane"
        )
        surface_layer = viewer.add_surface(mesh_data)

        # set up the slicing plane position
        volume_layer.plane = {"normal": [1, 0, 0], "position": [66, 187, 195]}

        # set up the camera
        viewer.camera.center = (64.0, 124.0, 127.5)
        viewer.camera.zoom = 3.87
        viewer.camera.angles = (
            -5.401480002668876,
            -0.16832643131442776,
            160.28901483338126,
        )

    else:
        # set up the viewer with the user-requested images
        if image_path is not None:
            # load the image if the path was passed
            image = mrcfile.read(image_path).astype(float)
            volume_layer = viewer.add_image(image)
        else:
            volume_layer = None

        if mesh_path is not None:
            # load the mesh if a path was passed
            mesh_data = read_obj_file(mesh_path)
            surface_layer = viewer.add_surface(mesh_data)
        else:
            surface_layer = None

    # Instantiate the widget and add it to Napari
    surforama_widget = QtSurforama(viewer, surface_layer, volume_layer)
    viewer.window.add_dock_widget(
        surforama_widget, area="right", name="Surforama"
    )

    napari.run()



import numpy as np
import trimesh


def read_obj_file(file_path):
    mesh = trimesh.load(file_path, file_type="obj", process=True)

    # Subdivide
    # verts, faces = trimesh.remesh.subdivide_to_size(
    # mesh.vertices, mesh.faces, 1
    # )

    # Subdivide can introduce holes
    # mesh = trimesh.Trimesh(vertices=verts, faces=faces)
    # trimesh.repair.fill_holes(mesh)

    verts = mesh.vertices
    faces = mesh.faces

    # trimesh swaps coordinate axes
    verts = verts[:, [2, 1, 0]]

    values = np.ones((len(verts),))

    return verts, faces, values
